import random
from PIL import Image, ImageDraw, ImageFont, ImageFilter
from .settings import api_settings as settings
from . import utils

try:
    from cStringIO import StringIO
except ImportError:
    from io import BytesIO as StringIO

FONT = ImageFont.truetype(
    settings.CAPTCHA_FONT_PATH, settings.CAPTCHA_FONT_SIZE)


def filter_default(image):
    return utils.filter_smooth(image, ImageFilter.SMOOTH)


def noise_default(image, draw):
    draw = utils.noise_dots(draw, image, settings.CAPTCHA_FOREGROUND_COLOR)
    draw = utils.noise_arcs(draw, image, settings.CAPTCHA_FOREGROUND_COLOR)


def getsize(font, text):
    if hasattr(font, 'getoffset'):
        return tuple(
            [x + y for x, y in zip(font.getsize(text), font.getoffset(text))])
    else:
        return font.getsize(text)


def makeimg(size):
    if settings.CAPTCHA_BACKGROUND_COLOR == "transparent":
        image = Image.new('RGBA', size)
    else:
        image = Image.new('RGB', size, settings.CAPTCHA_BACKGROUND_COLOR)
    return image


def generate_image(word):
    font = FONT
    size = settings.CAPTCHA_IMAGE_SIZE

    xpos = 2
    from_top = 4

    image = makeimg(size)

    for char in word:
        fgimage = Image.new('RGB', size, settings.CAPTCHA_FOREGROUND_COLOR)
        charimage = Image.new('L', getsize(font, ' %s ' % char), '#000000')
        chardraw = ImageDraw.Draw(charimage)
        chardraw.text((0, 0), ' %s ' % char, font=font, fill='#ffffff')
        if settings.CAPTCHA_LETTER_ROTATION:
            angle = random.randrange(*settings.CAPTCHA_LETTER_ROTATION)
            charimage = charimage.rotate(
                angle, expand=0, resample=Image.BICUBIC)

        charimage = charimage.crop(charimage.getbbox())
        maskimage = Image.new('L', size)

        xpos2 = xpos + charimage.size[0]
        from_top2 = from_top + charimage.size[1]
        maskimage.paste(charimage, (xpos, from_top, xpos2, from_top2))
        size = maskimage.size
        image = Image.composite(fgimage, image, maskimage)
        xpos = xpos + 2 + charimage.size[0]

    if settings.CAPTCHA_IMAGE_SIZE:
        # centering captcha on the image
        tmpimg = makeimg(size)
        xpos2 = int((size[0] - xpos) / 2)
        from_top2 = int((size[1] - charimage.size[1]) / 2 - from_top)
        tmpimg.paste(image, (xpos2, from_top2))
        image = tmpimg.crop((0, 0, size[0], size[1]))
    else:
        image = image.crop((0, 0, xpos + 1, size[1]))

    draw = ImageDraw.Draw(image)

    settings.FILTER_FUNCTION(image)
    settings.NOISE_FUNCTION(image, draw)

    out = StringIO()
    image.save(out, 'PNG')
    content = out.getvalue()
    out.seek(0)
    out.close()

    return content



from rest_framework import serializers
from django.utils.translation import ugettext as _
from django.core.cache import caches
from .settings import api_settings
from . import utils

cache = caches[api_settings.CAPTCHA_CACHE]


class RestCaptchaSerializer(serializers.Serializer):
    captcha_key = serializers.CharField(max_length=64)
    captcha_value = serializers.CharField(max_length=8, trim_whitespace=True)

    def validate(self, data):
        super(RestCaptchaSerializer, self).validate(data)
        cache_key = utils.get_cache_key(data['captcha_key'])

        real_value = cache.get(cache_key)

        if real_value is None:
            raise serializers.ValidationError(
                 _('Invalid or expared captcha key'))

        cache.delete(cache_key)
        if data['captcha_value'] != real_value:
            raise serializers.ValidationError(_('Invalid captcha value'))

        del data['captcha_key']
        del data['captcha_value']
        return data



from collections import namedtuple

VersionInfo = namedtuple('VersionInfo', ('major', 'minor', 'micro'))

VERSION = VersionInfo(0, 0, 2)

__version__ = '{0.major}.{0.minor}.{0.micro}'.format(VERSION)



import uuid
import base64
from rest_framework import views, response
from django.core.cache import caches
from .settings import api_settings
from . import utils
from . import captcha

cache = caches[api_settings.CAPTCHA_CACHE]


class RestCapthaView(views.APIView):
    def post(self, request):
        key = str(uuid.uuid4())
        value = utils.random_char_challenge(api_settings.CAPTCHA_LENGTH)
        cache_key = utils.get_cache_key(key)
        cache.set(cache_key, value, api_settings.CAPTCHA_TIMEOUT)

        # generate image
        image_bytes = captcha.generate_image(value)
        image_b64 = base64.b64encode(image_bytes)

        data = {
            api_settings.CAPTCHA_KEY: key,
            api_settings.CAPTCHA_IMAGE: image_b64,
            'image_type': 'image/png',
            'image_decode': 'base64'
        }
        return response.Response(data)



from setuptools import setup

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Operating System :: OS Independent',
    'Topic :: System :: Networking',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries',
    'Environment :: Console',
    'Intended Audience :: Developers',
]

dev_require = [
    'ipython',
]

tests_require = [
    'tox',
    'six',
]

install_requires = [
    'djangorestframework>3.5.0',
    'django',
    'Pillow==4.1.1',
]

entry_points = {
}

setup(
    author='Evgeny Zuev',
    author_email='zuevesn@gmail.com',
    name='django-rest-captcha',
    description='Simple captha for django rest framework',
    version='0.0.2',
    url='https://github.com/zueve/django-rest-captcha',
    platforms=['OS Independent'],
    install_requires=install_requires,
    extras_require={
        'tests': tests_require,
        'dev': dev_require,
    },
    package_dir={'rest_captcha': 'rest_captcha'},
    packages=['rest_captcha'],
    entry_points=entry_points,
    include_package_data=True,
    zip_safe=False,
    test_suite='',
)
